# -*- coding: utf-8 -*-
STATE_CANCELLED = "cancelled"
STATE_FINISHED = "finished"
STATE_MIXED = "mixed"
STATE_NEW = "new"
STATE_PROCESSING = "processing"
STATE_RESERVED = "reserved"

STATE_TRANSITION_CANCEL = "cancel"
STATE_TRANSITION_FINISH = "finish"
STATE_TRANSITION_PROCESS = "process"
STATE_TRANSITION_RENEW = "renew"

SALARIED_YES = "yes"
SALARIED_NO = "no"
SALARIED_MIXED = "mixed"
SALARIED_FAILED = "failed"

SALARIED_TRANSITION_SALARIED = "mark_salaried"
SALARIED_TRANSITION_OUTSTANDING = "mark_outstanding"
